import arcpy, ArcSAGA

class Toolbox(object):
	def __init__(self):
		self.label = "Classification"
		self.alias = ""
		self.tools = [tool_0, tool_1, tool_2, tool_4]

class tool_0(object):
	def __init__(self):
		self.label = "Supervised Image Classification"
		self.description = "<p>Standard methods for supervised image classification, including minimum distance, maximum likelihood, spectral angle mapping. Classifiers can be trained by areas defined through shapes, samples supplied as table records, or statistics previously stored to file. <p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B__hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="Features", name="GRIDS", direction="Input", parameterType="Required", datatype="GPRasterLayer", multiValue=True)
		params  = [param]
		param = arcpy.Parameter(displayName="Normalize", name="NORMALISE", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		param = arcpy.Parameter(displayName="Classification", name="CLASSES", direction="Output", parameterType="Required", datatype="GPRasterLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Quality", name="QUALITY", direction="Output", parameterType="Optional", datatype="GPRasterLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Look-up Table", name="CLASSES_LUT", direction="Output", parameterType="Optional", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Training", name="TRAIN_WITH", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["training areas", "training samples", "load from file"]
		param.value = "training areas"
		params += [param]
		param = arcpy.Parameter(displayName="Training Areas", name="TRAINING", direction="Input", parameterType="Required", datatype="GPFeatureLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Class Identifier", name="TRAINING_CLASS", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["TRAINING"]
		params += [param]
		param = arcpy.Parameter(displayName="Buffer Size", name="TRAIN_BUFFER", direction="Input", parameterType="Optional", datatype="GPDouble")
		param.value = 1.000000
		params += [param]
		param = arcpy.Parameter(displayName="Training Samples", name="TRAIN_SAMPLES", direction="Input", parameterType="Required", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Load Statistics from File...", name="FILE_LOAD", direction="Input", parameterType="Optional", datatype="DEFile")
		params += [param]
		param = arcpy.Parameter(displayName="Save Statistics to File...", name="FILE_SAVE", direction="Input", parameterType="Optional", datatype="DEFile")
		params += [param]
		param = arcpy.Parameter(displayName="Method", name="METHOD", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["Binary Encoding", "Parallelepiped", "Minimum Distance", "Mahalanobis Distance", "Maximum Likelihood", "Spectral Angle Mapping"]
		param.value = "Minimum Distance"
		params += [param]
		param = arcpy.Parameter(displayName="Distance Threshold", name="THRESHOLD_DIST", direction="Input", parameterType="Optional", datatype="GPDouble")
		param.value = 0.000000
		params += [param]
		param = arcpy.Parameter(displayName="Spectral Angle Threshold (Degree)", name="THRESHOLD_ANGLE", direction="Input", parameterType="Optional", datatype="GPDouble")
		param.value = 0.000000
		params += [param]
		param = arcpy.Parameter(displayName="Probability Threshold", name="THRESHOLD_PROB", direction="Input", parameterType="Optional", datatype="GPDouble")
		param.value = 0.000000
		params += [param]
		param = arcpy.Parameter(displayName="Probability Reference", name="RELATIVE_PROB", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["absolute", "relative"]
		param.value = "relative"
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('imagery_classification', '0')
		Tool.Set_Input ('GRIDS', parameters[0].valueAsText, 'grid_list')
		Tool.Set_Option('NORMALISE', parameters[1].valueAsText)
		Tool.Set_Output('CLASSES', parameters[2].valueAsText, 'grid')
		Tool.Set_Output('QUALITY', parameters[3].valueAsText, 'grid')
		Tool.Set_Output('CLASSES_LUT', parameters[4].valueAsText, 'table')
		Tool.Set_Option('TRAIN_WITH', parameters[5].valueAsText)
		Tool.Set_Input ('TRAINING', parameters[6].valueAsText, 'shapes')
		Tool.Set_Option('TRAINING_CLASS', parameters[7].valueAsText)
		Tool.Set_Option('TRAIN_BUFFER', parameters[8].valueAsText)
		Tool.Set_Input ('TRAIN_SAMPLES', parameters[9].valueAsText, 'table')
		Tool.Set_Option('FILE_LOAD', parameters[10].valueAsText)
		Tool.Set_Option('FILE_SAVE', parameters[11].valueAsText)
		Tool.Set_Option('METHOD', parameters[12].valueAsText)
		Tool.Set_Option('THRESHOLD_DIST', parameters[13].valueAsText)
		Tool.Set_Option('THRESHOLD_ANGLE', parameters[14].valueAsText)
		Tool.Set_Option('THRESHOLD_PROB', parameters[15].valueAsText)
		Tool.Set_Option('RELATIVE_PROB', parameters[16].valueAsText)
		Tool.Run()
		return


class tool_1(object):
	def __init__(self):
		self.label = "K-Means Clustering for Grids"
		self.description = "<p>This tool implements the K-Means cluster analysis for grids in two variants, iterative minimum distance (Forgy 1965) and hill climbing (Rubin 1967). <p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B__hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="Grids", name="GRIDS", direction="Input", parameterType="Required", datatype="GPRasterLayer", multiValue=True)
		params  = [param]
		param = arcpy.Parameter(displayName="Clusters", name="CLUSTER", direction="Output", parameterType="Required", datatype="GPRasterLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Statistics", name="STATISTICS", direction="Output", parameterType="Required", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Method", name="METHOD", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["Iterative Minimum Distance (Forgy 1965)", "Hill-Climbing (Rubin 1967)", "Combined Minimum Distance / Hillclimbing"]
		param.value = "Hill-Climbing (Rubin 1967)"
		params += [param]
		param = arcpy.Parameter(displayName="Clusters", name="NCLUSTER", direction="Input", parameterType="Optional", datatype="GPLong")
		param.value = 10
		params += [param]
		param = arcpy.Parameter(displayName="Maximum Iterations", name="MAXITER", direction="Input", parameterType="Optional", datatype="GPLong")
		param.value = 10
		params += [param]
		param = arcpy.Parameter(displayName="Normalise", name="NORMALISE", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		param = arcpy.Parameter(displayName="Start Partition", name="INITIALIZE", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["random", "periodical", "keep values"]
		param.value = "random"
		params += [param]
		param = arcpy.Parameter(displayName="Old Version", name="OLDVERSION", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('imagery_classification', '1')
		Tool.Set_Input ('GRIDS', parameters[0].valueAsText, 'grid_list')
		Tool.Set_Output('CLUSTER', parameters[1].valueAsText, 'grid')
		Tool.Set_Output('STATISTICS', parameters[2].valueAsText, 'table')
		Tool.Set_Option('METHOD', parameters[3].valueAsText)
		Tool.Set_Option('NCLUSTER', parameters[4].valueAsText)
		Tool.Set_Option('MAXITER', parameters[5].valueAsText)
		Tool.Set_Option('NORMALISE', parameters[6].valueAsText)
		Tool.Set_Option('INITIALIZE', parameters[7].valueAsText)
		Tool.Set_Option('OLDVERSION', parameters[8].valueAsText)
		Tool.Run()
		return


class tool_2(object):
	def __init__(self):
		self.label = "Confusion Matrix (Two Grids)"
		self.description = "<p>The tool allows one to compare two classified grids. It creates a confusion matrix and derived coefficients as well as the combinations of both classifications as new grid. The values of both grids must match each other in order to do the comparison.</p><p>The tool provides three options to define the grid classes:</p><p>- by providing a look-up table for each grid</p><p>- by coloring each grid with a look-up table (colors type = classified)  beforehand (only available in the GUI)</p><p>- by preparing the grid values appropriately; i.e., if no look-up table is provided, the tool simply extracts the classes from the grid values found in each grid</p><p>A typical application is a change detection analysis based on land cover classification of satellite imagery.</p><p><p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B__hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="Classification 1", name="ONE", direction="Input", parameterType="Required", datatype="GPRasterLayer")
		params  = [param]
		param = arcpy.Parameter(displayName="Look-up Table", name="ONE_LUT", direction="Input", parameterType="Optional", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Value", name="ONE_LUT_MIN", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["ONE_LUT"]
		params += [param]
		param = arcpy.Parameter(displayName="Value (Maximum)", name="ONE_LUT_MAX", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["ONE_LUT"]
		params += [param]
		param = arcpy.Parameter(displayName="Name", name="ONE_LUT_NAM", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["ONE_LUT"]
		params += [param]
		param = arcpy.Parameter(displayName="Classification 2", name="TWO", direction="Input", parameterType="Required", datatype="GPRasterLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Look-up Table", name="TWO_LUT", direction="Input", parameterType="Optional", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Value", name="TWO_LUT_MIN", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["TWO_LUT"]
		params += [param]
		param = arcpy.Parameter(displayName="Value (Maximum)", name="TWO_LUT_MAX", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["TWO_LUT"]
		params += [param]
		param = arcpy.Parameter(displayName="Name", name="TWO_LUT_NAM", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["TWO_LUT"]
		params += [param]
		param = arcpy.Parameter(displayName="Combined Classes", name="COMBINED", direction="Output", parameterType="Required", datatype="GPRasterLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Report Unchanged Classes", name="NOCHANGE", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		param = arcpy.Parameter(displayName="Include Unclassified Cells", name="NODATA", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		param = arcpy.Parameter(displayName="Confusion Matrix", name="CONFUSION", direction="Output", parameterType="Required", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Output as...", name="OUTPUT", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["cells", "percent", "area"]
		param.value = "cells"
		params += [param]
		param = arcpy.Parameter(displayName="Class Values", name="CLASSES", direction="Output", parameterType="Required", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Summary", name="SUMMARY", direction="Output", parameterType="Required", datatype="GPTableView")
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('imagery_classification', '2')
		Tool.Set_Input ('ONE', parameters[0].valueAsText, 'grid')
		Tool.Set_Input ('ONE_LUT', parameters[1].valueAsText, 'table')
		Tool.Set_Option('ONE_LUT_MIN', parameters[2].valueAsText)
		Tool.Set_Option('ONE_LUT_MAX', parameters[3].valueAsText)
		Tool.Set_Option('ONE_LUT_NAM', parameters[4].valueAsText)
		Tool.Set_Input ('TWO', parameters[5].valueAsText, 'grid')
		Tool.Set_Input ('TWO_LUT', parameters[6].valueAsText, 'table')
		Tool.Set_Option('TWO_LUT_MIN', parameters[7].valueAsText)
		Tool.Set_Option('TWO_LUT_MAX', parameters[8].valueAsText)
		Tool.Set_Option('TWO_LUT_NAM', parameters[9].valueAsText)
		Tool.Set_Output('COMBINED', parameters[10].valueAsText, 'grid')
		Tool.Set_Option('NOCHANGE', parameters[11].valueAsText)
		Tool.Set_Option('NODATA', parameters[12].valueAsText)
		Tool.Set_Output('CONFUSION', parameters[13].valueAsText, 'table')
		Tool.Set_Option('OUTPUT', parameters[14].valueAsText)
		Tool.Set_Output('CLASSES', parameters[15].valueAsText, 'table')
		Tool.Set_Output('SUMMARY', parameters[16].valueAsText, 'table')
		Tool.Run()
		return


class tool_4(object):
	def __init__(self):
		self.label = "Confusion Matrix (Polygons / Grid)"
		self.description = "<p>Compares a classified polygons layer with grid classes and creates a confusion matrix and derived coefficients. Grid classes have to be defined with a look-up table and values must match those of the polygon classes for the subsequent comparison. This tool is typically used for a quality assessment of a supervised classification. <p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B__hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="Classification", name="GRID", direction="Input", parameterType="Required", datatype="GPRasterLayer")
		params  = [param]
		param = arcpy.Parameter(displayName="Value Interpretation", name="GRID_VALUES", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["values are class identifiers", "use look-up table"]
		param.value = "use look-up table"
		params += [param]
		param = arcpy.Parameter(displayName="Look-up Table", name="GRID_LUT", direction="Input", parameterType="Optional", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Value", name="GRID_LUT_MIN", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["GRID_LUT"]
		params += [param]
		param = arcpy.Parameter(displayName="Value (Maximum)", name="GRID_LUT_MAX", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["GRID_LUT"]
		params += [param]
		param = arcpy.Parameter(displayName="Name", name="GRID_LUT_NAM", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["GRID_LUT"]
		params += [param]
		param = arcpy.Parameter(displayName="Polygons", name="POLYGONS", direction="Input", parameterType="Required", datatype="GPFeatureLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Classes", name="FIELD", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["POLYGONS"]
		params += [param]
		param = arcpy.Parameter(displayName="Confusion Matrix", name="CONFUSION", direction="Output", parameterType="Required", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Class Values", name="CLASSES", direction="Output", parameterType="Required", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Summary", name="SUMMARY", direction="Output", parameterType="Required", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Unclassified", name="NO_CLASS", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('imagery_classification', '6')
		Tool.Set_Input ('GRID', parameters[0].valueAsText, 'grid')
		Tool.Set_Option('GRID_VALUES', parameters[1].valueAsText)
		Tool.Set_Input ('GRID_LUT', parameters[2].valueAsText, 'table')
		Tool.Set_Option('GRID_LUT_MIN', parameters[3].valueAsText)
		Tool.Set_Option('GRID_LUT_MAX', parameters[4].valueAsText)
		Tool.Set_Option('GRID_LUT_NAM', parameters[5].valueAsText)
		Tool.Set_Input ('POLYGONS', parameters[6].valueAsText, 'shapes')
		Tool.Set_Option('FIELD', parameters[7].valueAsText)
		Tool.Set_Output('CONFUSION', parameters[8].valueAsText, 'table')
		Tool.Set_Output('CLASSES', parameters[9].valueAsText, 'table')
		Tool.Set_Output('SUMMARY', parameters[10].valueAsText, 'table')
		Tool.Set_Option('NO_CLASS', parameters[11].valueAsText)
		Tool.Run()
		return
