import arcpy, ArcSAGA

class Toolbox(object):
	def __init__(self):
		self.label = "ESRI E00"
		self.alias = ""
		self.tools = [tool_0]

class tool_0(object):
	def __init__(self):
		self.label = "Import ESRI E00 File"
		self.description = "<p>Import data sets from ESRI's E00 interchange format.</p><p></p><p>This import filter is based on the E00 format analysis of the GRASS GIS tool 'm.in.e00' written by Michel J. Wurtz. Go to the <a target=\"_blank\" href=\"http://grass.itc.it/\">GRASS GIS Hompage</a> for more information.</p><p>The <a target=\"_blank\" href=\"http://avce00.maptools.org/e00compr/index.html\">'E00Compr' library</a> written by Daniel Morissette has been used for e00 file access, so that compressed e00 files also can be read.</p><p><p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B__hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="Tables", name="TABLES", direction="Output", parameterType="Optional", datatype="GPTableView", multiValue=True)
		params  = [param]
		param = arcpy.Parameter(displayName="Shapes", name="SHAPES", direction="Output", parameterType="Optional", datatype="GPFeatureLayer", multiValue=True)
		params += [param]
		param = arcpy.Parameter(displayName="Grids", name="GRIDS", direction="Output", parameterType="Optional", datatype="GPRasterLayer", multiValue=True)
		params += [param]
		param = arcpy.Parameter(displayName="File", name="FILE", direction="Input", parameterType="Optional", datatype="DEFile", multiValue=True)
		params += [param]
		param = arcpy.Parameter(displayName="Import Extents", name="BBND", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		param = arcpy.Parameter(displayName="Import Tick Points", name="BTIC", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		param = arcpy.Parameter(displayName="Import Tables", name="BTABLES", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('io_esri_e00', '0')
		Tool.Set_Output('TABLES', parameters[0].valueAsText, 'table_list')
		Tool.Set_Output('SHAPES', parameters[1].valueAsText, 'shapes_list')
		Tool.Set_Output('GRIDS', parameters[2].valueAsText, 'grid_list')
		Tool.Set_Option('FILE', parameters[3].valueAsText)
		Tool.Set_Option('BBND', parameters[4].valueAsText)
		Tool.Set_Option('BTIC', parameters[5].valueAsText)
		Tool.Set_Option('BTABLES', parameters[6].valueAsText)
		Tool.Run()
		return
