import arcpy, ArcSAGA

class Toolbox(object):
	def __init__(self):
		self.label = "Image Classification"
		self.alias = ""
		self.tools = [tool_0, tool_1]

class tool_0(object):
	def __init__(self):
		self.label = "Local Climate Zone Classification"
		self.description = "<p>  Updates:</p><p>  <ul></p><p>  <li><b>2016/08/15</b> Automated filtering of Style Place Holders</li></p><p>  <li><b>2016/09/07</b> added cmd line support for kml import</li></p><p>  </ul>  </p><p> <p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B__hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="Features", name="FEATURES", direction="Input", parameterType="Required", datatype="GPRasterLayer", multiValue=True)
		params  = [param]
		param = arcpy.Parameter(displayName="grid definition for KML2shp conversion", name="GRIDDEFILE", direction="Input", parameterType="Optional", datatype="DEFile")
		params += [param]
		param = arcpy.Parameter(displayName="Training Areas", name="FILE_TRAINING", direction="Input", parameterType="Optional", datatype="DEFile")
		params += [param]
		param = arcpy.Parameter(displayName="Classifier", name="CLASSIFIER", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["Random Forest (ViGrA)", "Random Forest (OpenCV)", "Normal Bayes", "Support Vector Machine", "Artificial Neural Network"]
		param.value = "Random Forest (ViGrA)"
		params += [param]
		param = arcpy.Parameter(displayName="Class Definition", name="CLASS_DEF_SRC", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["built-in", "from file"]
		param.value = "built-in"
		params += [param]
		param = arcpy.Parameter(displayName="Class Definition File", name="CLASS_DEF_FILE", direction="Input", parameterType="Optional", datatype="DEFile")
		params += [param]
		param = arcpy.Parameter(displayName="LCZC", name="LCZC", direction="Output", parameterType="Required", datatype="GPRasterLayer")
		params += [param]
		param = arcpy.Parameter(displayName="KMZ File", name="LCZC_FILE", direction="Input", parameterType="Optional", datatype="DEFile")
		params += [param]
		param = arcpy.Parameter(displayName="LCZC (Filtered)", name="LCZC_FILTERED", direction="Output", parameterType="Optional", datatype="GPRasterLayer")
		params += [param]
		param = arcpy.Parameter(displayName="KMZ File", name="FILE_FILTERED_LCZC", direction="Input", parameterType="Optional", datatype="DEFile")
		params += [param]
		param = arcpy.Parameter(displayName="Majority Filter Radius", name="FILTER_RADIUS", direction="Input", parameterType="Optional", datatype="GPLong")
		param.value = 3
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('imagery_classification', 'lczc')
		Tool.Set_Input ('FEATURES', parameters[0].valueAsText, 'grid_list')
		Tool.Set_Option('GRIDDEFILE', parameters[1].valueAsText)
		Tool.Set_Option('FILE_TRAINING', parameters[2].valueAsText)
		Tool.Set_Option('CLASSIFIER', parameters[3].valueAsText)
		Tool.Set_Option('CLASS_DEF_SRC', parameters[4].valueAsText)
		Tool.Set_Option('CLASS_DEF_FILE', parameters[5].valueAsText)
		Tool.Set_Output('LCZC', parameters[6].valueAsText, 'grid')
		Tool.Set_Option('LCZC_FILE', parameters[7].valueAsText)
		Tool.Set_Output('LCZC_FILTERED', parameters[8].valueAsText, 'grid')
		Tool.Set_Option('FILE_FILTERED_LCZC', parameters[9].valueAsText)
		Tool.Set_Option('FILTER_RADIUS', parameters[10].valueAsText)
		Tool.Run()
		return


class tool_1(object):
	def __init__(self):
		self.label = "Supervised Majority Choice Image Classification"
		self.description = "<p>  The majority choice tool for supervised image classification runs the selected classification tools using standard settings and takes for each pixel of the resulting classification the class that has been identified most often by the individual classifiers.</p><p></p><p>  <i>Random Forest</i> is not selected by default because it generates randomized results with more or less strong differences each time it is run.</p><p>  <i>K-Nearest Neighbours</i> and <i>Artifical Neural Network</i> have been excluded from default, because using them might be a bit more time consuming.</p><p> <p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B__hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="Features", name="FEATURES", direction="Input", parameterType="Required", datatype="GPRasterLayer", multiValue=True)
		params  = [param]
		param = arcpy.Parameter(displayName="Normalize", name="NORMALIZE", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		param = arcpy.Parameter(displayName="Training", name="MODEL_TRAIN", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["training areas", "training samples"]
		param.value = "training areas"
		params += [param]
		param = arcpy.Parameter(displayName="Training Areas", name="TRAIN_AREAS", direction="Input", parameterType="Required", datatype="GPFeatureLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Class Identifier", name="TRAIN_CLASS", direction="Input", parameterType="Optional", datatype="Field")
		param.parameterDependencies = ["TRAIN_AREAS"]
		params += [param]
		param = arcpy.Parameter(displayName="Buffer Size", name="TRAIN_BUFFER", direction="Input", parameterType="Optional", datatype="GPDouble")
		param.value = 30.000000
		params += [param]
		param = arcpy.Parameter(displayName="Training Samples", name="TRAIN_SAMPLES", direction="Input", parameterType="Required", datatype="GPTableView")
		params += [param]
		param = arcpy.Parameter(displayName="Majority Choice", name="CLASSES", direction="Output", parameterType="Required", datatype="GPRasterLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Majority Count", name="MAJORITY_COUNT", direction="Output", parameterType="Optional", datatype="GPRasterLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Classes Count", name="NUNIQUES", direction="Output", parameterType="Optional", datatype="GPRasterLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Unambiguous", name="UNAMBIGUOUS", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		param = arcpy.Parameter(displayName="Parallel Epiped", name="CLASSIFY_BOX", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		param = arcpy.Parameter(displayName="Minimum Distance", name="CLASSIFY_MINDIST", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		param = arcpy.Parameter(displayName="Mahalonobis Distance", name="CLASSIFY_MAHALONOBIS", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		param = arcpy.Parameter(displayName="Maximum Likelihood", name="CLASSIFY_MAXLIKE", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		param = arcpy.Parameter(displayName="Spectral Angle Mapping", name="CLASSIFY_SAM", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		param = arcpy.Parameter(displayName="Normal Bayes", name="CLASSIFY_BAYES", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		param = arcpy.Parameter(displayName="Decision Tree", name="CLASSIFY_DT", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		param = arcpy.Parameter(displayName="Random Forest", name="CLASSIFY_RF", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		param = arcpy.Parameter(displayName="Support Vector Machine", name="CLASSIFY_SVM", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = True
		params += [param]
		param = arcpy.Parameter(displayName="K-Nearest Neighbours", name="CLASSIFY_KNN", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		param = arcpy.Parameter(displayName="Artificial Neural Network", name="CLASSIFY_ANN", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('imagery_classification', 'classify_majority')
		Tool.Set_Input ('FEATURES', parameters[0].valueAsText, 'grid_list')
		Tool.Set_Option('NORMALIZE', parameters[1].valueAsText)
		Tool.Set_Option('MODEL_TRAIN', parameters[2].valueAsText)
		Tool.Set_Input ('TRAIN_AREAS', parameters[3].valueAsText, 'shapes')
		Tool.Set_Option('TRAIN_CLASS', parameters[4].valueAsText)
		Tool.Set_Option('TRAIN_BUFFER', parameters[5].valueAsText)
		Tool.Set_Input ('TRAIN_SAMPLES', parameters[6].valueAsText, 'table')
		Tool.Set_Output('CLASSES', parameters[7].valueAsText, 'grid')
		Tool.Set_Output('MAJORITY_COUNT', parameters[8].valueAsText, 'grid')
		Tool.Set_Output('NUNIQUES', parameters[9].valueAsText, 'grid')
		Tool.Set_Option('UNAMBIGUOUS', parameters[10].valueAsText)
		Tool.Set_Option('CLASSIFY_BOX', parameters[11].valueAsText)
		Tool.Set_Option('CLASSIFY_MINDIST', parameters[12].valueAsText)
		Tool.Set_Option('CLASSIFY_MAHALONOBIS', parameters[13].valueAsText)
		Tool.Set_Option('CLASSIFY_MAXLIKE', parameters[14].valueAsText)
		Tool.Set_Option('CLASSIFY_SAM', parameters[15].valueAsText)
		Tool.Set_Option('CLASSIFY_BAYES', parameters[16].valueAsText)
		Tool.Set_Option('CLASSIFY_DT', parameters[17].valueAsText)
		Tool.Set_Option('CLASSIFY_RF', parameters[18].valueAsText)
		Tool.Set_Option('CLASSIFY_SVM', parameters[19].valueAsText)
		Tool.Set_Option('CLASSIFY_KNN', parameters[20].valueAsText)
		Tool.Set_Option('CLASSIFY_ANN', parameters[21].valueAsText)
		Tool.Run()
		return
