import arcpy, ArcSAGA

class Toolbox(object):
	def __init__(self):
		self.label = "Image Segmentation"
		self.alias = ""
		self.tools = [tool_0]

class tool_0(object):
	def __init__(self):
		self.label = "Object Based Image Segmentation"
		self.description = "<p>  This <i>Object Based Image Segmentation</i> tool chain combines a number of tools for an easy derivation of geo-objects as polygons and is typically applied to satellite imagery. Segmentation is done using a 'Seeded Region Growing Algorithm'. Optionally the resulting polygons can be grouped by an unsupervised classification (k-means cluster analysis) or supervised classification (needs classified feature samples as additional input), both is performed on the basis of zonal feature grid statistics for each polygon object.</p><p> <p><hr><span STYLE=\"font-style:italic;font-size:9pt;font-weight:bold\">SAGA - System for Automated Geoscientific Analyses<br></span><span STYLE=\"font-style:italic;font-size:8pt\">www.saga-gis.org<br></span><span STYLE=\"font-style:italic;font-size:6pt\">_____<br>Reference:<br>Conrad, O., Bechtel, B., Bock, M., Dietrich, H., Fischer, E., Gerlitz, L., Wehberg, J., Wichmann, V., and B__hner, J. (2015): System for Automated Geoscientific Analyses (SAGA) v. 2.1.4, Geosci. Model Dev., 8, 1991-2007, doi:10.5194/gmd-8-1991-2015.</span></p></p>"
		self.canRunInBackground = False

	def getParameterInfo(self):
		param = arcpy.Parameter(displayName="Features", name="FEATURES", direction="Input", parameterType="Required", datatype="GPRasterLayer", multiValue=True)
		params  = [param]
		param = arcpy.Parameter(displayName="Normalize", name="NORMALIZE", direction="Input", parameterType="Optional", datatype="GPBoolean")
		param.value = False
		params += [param]
		param = arcpy.Parameter(displayName="Segments", name="OBJECTS", direction="Output", parameterType="Required", datatype="GPFeatureLayer")
		params += [param]
		param = arcpy.Parameter(displayName="Band Width for Seed Point Generation", name="SEEDS_BAND_WIDTH", direction="Input", parameterType="Optional", datatype="GPDouble")
		param.value = 2.000000
		params += [param]
		param = arcpy.Parameter(displayName="Neighbourhood", name="RGA_NEIGHBOUR", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["4 (Neumann)", "8 (Moore)"]
		param.value = "4 (Neumann)"
		params += [param]
		param = arcpy.Parameter(displayName="Distance", name="RGA_METHOD", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["feature space and position", "feature space"]
		param.value = "feature space"
		params += [param]
		param = arcpy.Parameter(displayName="Variance in Feature Space", name="RGA_SIG_1", direction="Input", parameterType="Optional", datatype="GPDouble")
		param.value = 1.000000
		params += [param]
		param = arcpy.Parameter(displayName="Variance in Position Space", name="RGA_SIG_2", direction="Input", parameterType="Optional", datatype="GPDouble")
		param.value = 1.000000
		params += [param]
		param = arcpy.Parameter(displayName="Similarity Threshold", name="RGA_SIMILARITY", direction="Input", parameterType="Optional", datatype="GPDouble")
		param.value = 0.000000
		params += [param]
		param = arcpy.Parameter(displayName="Generalization", name="MAJORITY_RADIUS", direction="Input", parameterType="Optional", datatype="GPLong")
		param.value = 1
		params += [param]
		param = arcpy.Parameter(displayName="Classification", name="CLASSIFICATION", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["none", "cluster analysis", "supervised classification"]
		param.value = "none"
		params += [param]
		param = arcpy.Parameter(displayName="Split Distinct Polygon Parts", name="SPLIT_POLYGONS", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["no", "yes"]
		param.value = "yes"
		params += [param]
		param = arcpy.Parameter(displayName="Number of Clusters", name="NCLUSTER", direction="Input", parameterType="Optional", datatype="GPLong")
		param.value = 10
		params += [param]
		param = arcpy.Parameter(displayName="Method", name="CLASSIFIER", direction="Input", parameterType="Optional", datatype="GPString")
		param.filter.list = ["Binary Encoding", "Parallelepiped", "Minimum Distance", "Mahalanobis Distance", "Maximum Likelihood", "Spectral Angle Mapping"]
		param.value = "Minimum Distance"
		params += [param]
		param = arcpy.Parameter(displayName="Training Samples", name="SAMPLES", direction="Input", parameterType="Required", datatype="GPTableView")
		params += [param]
		return params

	def execute(self, parameters, messages):
		Tool = ArcSAGA.SAGA_Tool('imagery_segmentation', 'obia')
		Tool.Set_Input ('FEATURES', parameters[0].valueAsText, 'grid_list')
		Tool.Set_Option('NORMALIZE', parameters[1].valueAsText)
		Tool.Set_Output('OBJECTS', parameters[2].valueAsText, 'shapes')
		Tool.Set_Option('SEEDS_BAND_WIDTH', parameters[3].valueAsText)
		Tool.Set_Option('RGA_NEIGHBOUR', parameters[4].valueAsText)
		Tool.Set_Option('RGA_METHOD', parameters[5].valueAsText)
		Tool.Set_Option('RGA_SIG_1', parameters[6].valueAsText)
		Tool.Set_Option('RGA_SIG_2', parameters[7].valueAsText)
		Tool.Set_Option('RGA_SIMILARITY', parameters[8].valueAsText)
		Tool.Set_Option('MAJORITY_RADIUS', parameters[9].valueAsText)
		Tool.Set_Option('CLASSIFICATION', parameters[10].valueAsText)
		Tool.Set_Option('SPLIT_POLYGONS', parameters[11].valueAsText)
		Tool.Set_Option('NCLUSTER', parameters[12].valueAsText)
		Tool.Set_Option('CLASSIFIER', parameters[13].valueAsText)
		Tool.Set_Input ('SAMPLES', parameters[14].valueAsText, 'table')
		Tool.Run()
		return
